# Copyright (c) 2011 Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program. If not, see
# <http://www.gnu.org/licenses/>.

"""Test helpers for working with cassandra."""

import os.path

from fixtures import Fixture, TempDir
from pycassa.system_manager import SystemManager

from grackle.server.cassandra import workaround_1779
from grackle.server.model import create_schema


class TemporaryKeyspace(Fixture):
    """Create a temporary keyspace.

    The keyspace is named after a tempdir.
    """

    def setUp(self):
        super(TemporaryKeyspace, self).setUp()
        tempdir = self.useFixture(TempDir())
        self.keyspace = os.path.basename(tempdir.path)
        self.mgr = SystemManager()
        workaround_1779(self.mgr.create_keyspace, self.keyspace,
            replication_factor=1)
        self.addCleanup(workaround_1779, self.mgr.drop_keyspace,
            self.keyspace)


class TemporaryDB(Fixture):
    """Create a temporary usable OOPS DB.

    The keyspace for it is at self.keyspace.
    """

    def setUp(self):
        super(TemporaryDB, self).setUp()
        self.keyspace = self.useFixture(TemporaryKeyspace()).keyspace
        create_schema('localhost', self.keyspace)
